/*****************************************************************************
* amf_usb_dahdi.c  Amfeltec USB Driver
* 		
* December 2013		
*****************************************************************************/

#ifndef __AMF_USB_DAHDI__
#define __AMF_USB_DAHDI__

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/usb.h>
#include "amf_usb.h"
#include "amf_usb_utils.h"

//================================================================ HOOKSIG ================

static int amf_ioctl(struct dahdi_chan *chan, unsigned int cmd, unsigned long data)
{

	switch(cmd) {
	case DAHDI_ONHOOKTRANSFER:
		break;
	case DAHDI_SETPOLARITY:
		break;
	case DAHDI_TONEDETECT:
		// Chan_dahdi asking us about our tonedetect mode.
		// We should answer (no HW Tone Support, so i->hardwaredtmf = 0 => dahdi_pvt->dsp = yes!):
		return -ENOSYS;
	default:
		return -ENOTTY;
	}
	return 0;
}


static int amf_hooksig(struct dahdi_chan *chan, enum dahdi_txsig txsig)
{
	amf_usb_t *ausb = chan->pvt;
	int err = 0;

	switch(txsig) {
	case DAHDI_TXSIG_START:
		break;
	case DAHDI_TXSIG_OFFHOOK:
		A_DEBUG("%s%d: goes off-hook (txsig %d)\n", AMF_USB_NAME, ausb->order, txsig);
		ausb->fx[chan->chanpos - 1].fxo.offhook = 1;						
		err = __amf_usb_fxo_write(ausb, chan->chanpos - 1, 5, 0x9);
		break;
	case DAHDI_TXSIG_ONHOOK:
		A_DEBUG("%s%d: goes on-hook (txsig %d)\n", AMF_USB_NAME, ausb->order, txsig);
		ausb->fx[chan->chanpos - 1].fxo.offhook = 0;						
		err = __amf_usb_fxo_write(ausb, chan->chanpos - 1, 5, 0x8);
		break;
	default:
		A_DEBUG("%s%d: Can't set tx state to %d (chan %d)\n",	AMF_USB_NAME, ausb->order, txsig, chan->chanpos);
	}
	return 0;
}


static int amf_watchdog(struct dahdi_span *span, int event)
{
	return 0;
}

void amf_dahdi_sync_tick(struct dahdi_span *span, int is_master)
{
	
	struct dahdi_chan *chan = 	span->chans[0];	
	amf_usb_t *ausb = 			chan->pvt;


	ausb->dahdi_ticks++;

	return;
}

static int amf_open(struct dahdi_chan *chan)
{
	amf_usb_t *ausb = chan->pvt;
	if (ausb->dead)
		return -ENODEV;
	ausb->usecount++;
	A_DEBUG("%s%d: Open (usecount=%d, channo=%d, chanpos=%d)\n", AMF_USB_NAME, ausb->order,	ausb->usecount, chan->channo, chan->chanpos);
	return 0;
}

static int amf_close(struct dahdi_chan *chan)
{
	amf_usb_t *ausb = chan->pvt;
	ausb->usecount--;
	return 0;
}

static const struct dahdi_span_ops amf_usb_ops = {
#if DAHDI_VERSION >= 24
	.owner = THIS_MODULE,
#endif
	.open = amf_open,
	.close = amf_close,
	.hooksig = amf_hooksig,
	.ioctl = amf_ioctl,
	.watchdog =	amf_watchdog,
	.sync_tick = amf_dahdi_sync_tick,
};

int amf_usb_dahdi_register(amf_usb_t *ausb) {

	int err;


	if (ausb->amf_device_type == AMF_USB_FXO) {
		sprintf(ausb->span.name, "AMF_USB_FXO_%d", ausb->order);
		sprintf(ausb->span.desc, "AMF_USB_FXO %d", ausb->order);		
#if DAHDI_VERSION >= 27
		ausb->span.spantype = SPANTYPE_ANALOG_FXO;
#endif
	} else {
		sprintf(ausb->span.name, "AMF_USB_FXS_%d", ausb->order);
		sprintf(ausb->span.desc, "AMF_USB_FXS %d", ausb->order);
#if DAHDI_VERSION >= 27
		ausb->span.spantype = SPANTYPE_ANALOG_FXS;
#endif
	}

	/*snprintf(ausb->span.desc, sizeof(ausb->span.desc) - 1, "%s %d", ausb->span.name, ausb->order);*/

#if DAHDI_VERSION <= 25
	ausb->span.manufacturer	= "Amfeltec";
#endif
	ausb->chan_ptrs[0] = &ausb->chans[0];

	if (ausb->amf_device_type == AMF_USB_FXO) {
		sprintf(ausb->chans[0].name, "AMF_USB_FXO/%d/%d", ausb->order, 1);

		ausb->chans[0].sigcap =	DAHDI_SIG_FXSKS |
								DAHDI_SIG_FXSLS |
								DAHDI_SIG_SF    |
								DAHDI_SIG_CLEAR;

	} else {
		sprintf(ausb->chans[0].name, "AMF_USB_FXS/%d/%d", ausb->order, 1);

		ausb->chans[0].sigcap =	DAHDI_SIG_FXOKS |
								DAHDI_SIG_FXOLS |
								DAHDI_SIG_FXOGS |
								DAHDI_SIG_SF    |
								DAHDI_SIG_EM    |
								DAHDI_SIG_CLEAR;
	}

#if DAHDI_VERSION >= 26
	ausb->ddev = dahdi_create_device();
	dev_set_name(&ausb->ddev->dev, ausb->device_name);
	ausb->ddev->manufacturer = "Amfeltec";
	ausb->ddev->devicetype = "Amfeltec USB device";
	ausb->ddev->location = kasprintf(GFP_KERNEL, "USB bus");
#endif

	ausb->chans[0].chanpos = 1;
	ausb->chans[0].pvt = ausb;
	ausb->span.chans 		= ausb->chan_ptrs;
	ausb->span.channels 	= 1;							

#if DAHDI_VERSION == 23
	dahdi->span.owner = THIS_MODULE;
#endif	
#if DAHDI_VERSION < 25
	init_waitqueue_head(&ausb->span.maintq);
#endif
#if DAHDI_VERSION < 24
	ausb->span.pvt = ausb;
#endif

	if (ausb->alawoverride) {
		ausb->span.deflaw 		= DAHDI_LAW_ALAW;			// Europe
	}
	else {
		ausb->span.deflaw 		= DAHDI_LAW_MULAW;			// America
	}

		
	ausb->battthresh		= DEFAULT_BATT_THRESH;
	ausb->battdebounce		= DEFAULT_BATT_DEBOUNCE;

#if DAHDI_VERSION >= 24		
	ausb->span.ops			= &amf_usb_ops;
#endif

	ausb->span.flags		= DAHDI_FLAG_RBS;           // Robbed-bit signalling, otherwise no hooksigs!

#if DAHDI_VERSION >= 26
	list_add_tail(&ausb->span.device_node, &ausb->ddev->spans);
	err = dahdi_register_device(ausb->ddev, NULL);
#else
	err = dahdi_register(&ausb->span, 0);
#endif

	return err;

}


int amf_usb_dahdi_unregister(amf_usb_t *ausb) {

#if DAHDI_VERSION >= 26
	dahdi_unregister_device(ausb->ddev);
#else
	dahdi_unregister(&ausb->span);
#endif

	return 0;

}



#endif  //__AMF_USB_DAHDI__
