/*****************************************************************************
* amf_usb.h  Amfeltec USB Driver
* 		
* December 2013
******************************************************************************/


#ifndef __AMF_USB_H__
#define __AMF_USB_H__

#include "amf_usb_utils.h"
#include "dahdi/kernel.h"

//=================================================================================

#define AMF_USB_NAME					"amf_usb"
#define AMF_USB_VENDORID				0x10C4
#define AMF_USB_PRODUCTID_FXO			0x88F4			
#define AMF_USB_PRODUCTID_FXS			0x88F6			
#define AMF_USB_PRODUCTID_PA			0x88F8			
	
 /* settings for the 'adapter_type' */
enum {
         F100_ADAPTER = 0x01,	/* F100 */
         F120_ADAPTER			/* F120 */
};

#define AMF_USB_FXO						1
#define AMF_USB_FXS						2
#define AMF_USB_PA						3

#define AMF_USBFXO_SYNC_DELAY			10
#define AMF_USBFXO_SYNC_RX_RETRIES		100
#define AMF_USBFXO_SYNC_TX_RETRIES		400
#define AMF_USBFXO_READ_DELAY			20             
#define AMF_USBFXO_READ_RETRIES			100            
#define AMF_USBFXO_WRITE_DELAY			10

#define AMF_USB_STATUS_ATTACHED			1
#define AMF_USB_STATUS_READY			2
#define AMF_USB_STATUS_TX_READY			3
#define AMF_USB_STATUS_RX_EVENT1_READY	4
#define AMF_USB_STATUS_RX_EVENT2_READY	5
#define AMF_USB_STATUS_RX_DATA_READY	6
#define AMF_USB_STATUS_BH				7
#define AMF_USB_STATUS_TX_CMD			8

#define AMF_USB_LSTATUS_GETDEV			1
#define AMF_USB_LSTATUS_TASKLET			2
#define AMF_USB_LSTATUS_WRITEBUF		3
#define AMF_USB_LSTATUS_READBUF			4
#define AMF_USB_LSTATUS_RXCMD			5
#define AMF_USB_LSTATUS_TXCMD			6
#define AMF_USB_LSTATUS_INTF			7
#define AMF_USB_LSTATUS_URB				8
#define AMF_USB_LSTATUS_DAHDI			9
#define AMF_USB_LSTATUS_MISC			10


#define AMF_USB_STATUS_FE_RW			1

#define AMF_URB_STATUS_READY			1

#define AMF_USB_IDLE_PATTERN			0x7E
#define AMF_USB_CTRL_IDLE_PATTERN		0x7E

#define AMF_USB_MAX_RX_CMD_QLEN			20	
#define AMF_USB_MAX_TX_CMD_QLEN			20	



//=================================================================================

#define MAX_USB_DEVICES			128				
#define MAX_USB_MODULES			1				

#define AMF_USB_CHUNKSIZE		8
#define AMF_USB_FRAMESCALE		1
#define AMF_USB_RXTX_CHUNKSIZE	(AMF_USB_CHUNKSIZE*AMF_USB_FRAMESCALE)	// 8
#define AMF_USB_MAX_CHUNKSIZE	(AMF_USB_CHUNKSIZE*AMF_USB_FRAMESCALE)	// 8

#define MAX_READ_URB_COUNT		1										// 1
#define MAX_WRITE_URB_COUNT		2										// 2
#define MAX_USB_RX_LEN			(4*AMF_USB_RXTX_CHUNKSIZE)              // 32
#define MAX_USB_TX_LEN			(4*AMF_USB_RXTX_CHUNKSIZE)              // 32
#define MAX_READ_BUF_LEN		(MAX_USB_RX_LEN*4)                    	// 32*4, 
#define MAX_WRITE_BUF_LEN		(MAX_USB_TX_LEN*4)    	                // 32*4, 
#define MAX_USB_CTRL_CMD_LEN	(AMF_USB_FRAMESCALE*8)	                // 16		/* bytes */




//====================================================== Internal USB-FXO CPU registers
#define AMF_USB_CPU_REG_DEVICEID				0x00
#define AMF_USB_DEVICEID						0x55

#define AMF_USB_CPU_REG_HARDWAREVER				0x01
#define AMF_USB_CPU_REG_FIRMWAREVER				0x02

#define AMF_USB_CPU_REG_CTRL			0x03
#define AMF_USB_CPU_BIT_CTRL_RESET		0x80
#define AMF_USB_CPU_BIT_CTRL_FWUPDATE		0x40
#define AMF_USB_CPU_BIT_CTRL_TS1_HWEC_EN	0x08
#define AMF_USB_CPU_BIT_CTRL_TS0_HWEC_EN	0x04
#define AMF_USB_CPU_BIT_CTRL_TS1_EVENT_EN	0x02
#define AMF_USB_CPU_BIT_CTRL_TS0_EVENT_EN	0x01

#define AMF_USB_CPU_REG_FIFO_STATUS		0x04
#define AMF_USB_CPU_BIT_FIFO_STATUS_TS1_TX_UF	0x80
#define AMF_USB_CPU_BIT_FIFO_STATUS_TS1_TX_OF	0x40
#define AMF_USB_CPU_BIT_FIFO_STATUS_TS0_TX_UF	0x20
#define AMF_USB_CPU_BIT_FIFO_STATUS_TS0_TX_OF	0x10
#define AMF_USB_CPU_BIT_FIFO_STATUS_TS1_RX_UF	0x08
#define AMF_USB_CPU_BIT_FIFO_STATUS_TS1_RX_OF	0x04
#define AMF_USB_CPU_BIT_FIFO_STATUS_TS0_RX_UF	0x02
#define AMF_USB_CPU_BIT_FIFO_STATUS_TS0_RX_OF	0x01

#define AMF_USB_CPU_REG_UART_STATUS		0x05
#define AMF_USB_CPU_BIT_UART_STATUS_LOST_SYNC	0x10
#define AMF_USB_CPU_BIT_UART_STATUS_CMD_UNKNOWN	0x10
#define AMF_USB_CPU_BIT_UART_STATUS_RX_UF	0x08
#define AMF_USB_CPU_BIT_UART_STATUS_RX_OF	0x04
#define AMF_USB_CPU_BIT_UART_STATUS_TX_UF	0x02
#define AMF_USB_CPU_BIT_UART_STATUS_TX_OF	0x01

#define AMF_USB_CPU_REG_HOSTIF_STATUS		0x06
#define AMF_USB_CPU_BIT_HOSTIF_STATUS_RX_UF	0x08
#define AMF_USB_CPU_BIT_HOSTIF_STATUS_RX_OF	0x04
#define AMF_USB_CPU_BIT_HOSTIF_STATUS_TX_UF	0x02
#define AMF_USB_CPU_BIT_HOSTIF_STATUS_TX_OF	0x01

#define AMF_USB_CPU_REG_LED_CONTROL		0x07
#define AMF_USB_CPU_BIT_LED_CONTROL_TS1_GRN	0x08
#define AMF_USB_CPU_BIT_LED_CONTROL_TS1_RED	0x04
#define AMF_USB_CPU_BIT_LED_CONTROL_TS0_GRN	0x02
#define AMF_USB_CPU_BIT_LED_CONTROL_TS0_RED	0x01

#define AMF_USB_CPU_REG_DEBUG			0x08
#define AMF_USB_CPU_BIT_DEBUG_WEN_ACK		0x08
#define AMF_USB_CPU_BIT_DEBUG_DTMF		0x04
#define AMF_USB_CPU_BIT_DEBUG_LOCAL_LB		0x02
#define AMF_USB_CPU_BIT_DEBUG_LINE_LB		0x01

#define AMF_USB_CPU_REG_EC_NUM			0x09


//====================================================== FXO

#define RING_DEBOUNCE           16     /* Ringer Debounce (64 ms) */
#define DEFAULT_BATT_DEBOUNCE   16     /* Battery debounce (64 ms) */
#define POLARITY_DEBOUNCE       16     /* Polarity debounce (64 ms) */
#define DEFAULT_BATT_THRESH     3      /* Anything under this is "no battery" */
#define DEFAULT_OH_THRESH       10      /* Anything under this is "off-hook" */
#define OHT_TIMER               6000    /* How long after RING to retain OHT */

#define FXO_LINK_DEBOUNCE       400		//200 (as for Polarity Reverse Issue)

#define MAX_ALARMS              10
#define FXO_LINK_THRESH         1      /* fxo link threshold */

#define FXO_DISCONNECTED		0x01
#define FXO_CONNECTED			0x02



struct amf_urb {
	void			*pvt;
	int				id;
	int				next_off;
	unsigned int	ready;
	struct urb		urb;
};


typedef struct sk_buff_head	amf_skb_queue_t;


//====================================================  STATS
typedef struct {
	int		core_notready_cnt;
	int		cmd_overrun;
	int		cmd_timeout;
	int		cmd_invalid;
	int		rx_sync_err_cnt;
	int		rx_start_fr_err_cnt;
	int		rx_start_err_cnt;
	int		rx_cmd_reset_cnt;
	int		rx_cmd_drop_cnt;
	int		rx_cmd_unknown;
	int		rx_overrun_cnt;
	int		rx_underrun_cnt;
	int		tx_overrun_cnt;
	int		tx_notready_cnt;
	int		tx_endpointstall_cnt;
	int		rx_endpointstall_cnt;
	unsigned char	dev_fifo_status;
	unsigned char	dev_uart_status;
	unsigned char	dev_hostif_status;
	int		dev_sync_err_cnt;
} amf_usb_comm_err_stats_t;

//====================================================  
struct amf_usb_desc {
        char	*name;
        int	adptr_type;
};


typedef struct fxo_
{
	int     ready;
	u8      status;                 /* line status (connected/disconnected) */
    int     statusdebounce;         /* line status debounce */
    int     ring_detect;
    int     offhook;                /* Xswitch */
	int		onhook;					// ZERO_BATT_RING special	
    int     battery;                /* Xswitch */
    int     battdebounce;           /* Xswitch */
    int		i_debounce;
    int		ringdebounce;
    int     wasringing;
    int     lastpol;
    int     polarity;
    int     polaritydebounce;
    /* Variables required for tapper mode */
	int     ohdebounce;
	int     going_offhook;  /* current ohdebounce is for off-hk or on-hk */
	unsigned char   imask;          /* interrupt mask */
	int                             readcid;
	unsigned int                    cidtimer;
    /*Additional for Zaptel mode*/
	int                             echotune;       /* echo tune */
	//struct wan_rm_echo_coefs        echoregs;       /* echo tune */
} amf_fxo_t;


typedef struct fxs_ 
{
        int     ready;
        int     lasttxhook;
        int     lasttxhook_update;
        int     lastrxhook;
        int     oldrxhook;
        int     debouncehook;
        int     debounce;
        int     palarms;
        int     ohttimer;
} amf_fxs_t;


//===============================================================================================  amf_usb_t
typedef struct amf_usb
{
	
	struct usb_device		*udev;
	struct usb_interface 	*uintf;	
	int						amf_device_type;				// FXO/FXS...
	u8						partnum;						// CP210X_PARTNUM
	unsigned int			num;							// amf_usb device number
	int						order;							// by rotary switch
	int						firmwareupdate;					// if not 0: firmware update mode 
	int						failover_enabled;				// hardware version with fail-over relay
	char					bus_id[20];
	unsigned long			lstatus;						// load status
	unsigned int			status;
	unsigned int			fe_status[2];
	unsigned int			mod_cnt;

	struct dahdi_span 		span;
	struct dahdi_chan 		chans[MAX_USB_MODULES];
	struct dahdi_chan 		*chan_ptrs[MAX_USB_MODULES];
#if DAHDI_VERSION >= 26
	struct dahdi_device		*ddev;
#endif
	char   					device_name[20];		

	unsigned char 			lasttx[MAX_USB_MODULES][DAHDI_CHUNKSIZE];

	int 					mod_type[MAX_USB_MODULES];		// FXO/FXS...
	union {
			amf_fxo_t		fxo;
			amf_fxs_t		fxs;
	} fx[MAX_USB_MODULES];
	
	int						alawoverride;					// global for FXO
	int						opermode;
	int						battdebounce;			
	int						battthresh;				
	int						battalarm;				
	int						fxo_txgain;
	int						fxo_rxgain;
	
	int						dead;
	int						usecount;

	struct amf_urb			urbread[MAX_READ_URB_COUNT];		// RX URB
	struct amf_urb			urbwrite[MAX_WRITE_URB_COUNT];		// TX URB
	#define	urbread_to_usb(urb) container_of(urb, struct amf_usb, urbread)
	#define	urbwrite_to_usb(urb) container_of(urb, struct amf_usb, urbread)

	char					*readbuf;							// RX buffer
	char					*writebuf;							// TX buffer
	char					readbuf_block[64];

	unsigned char	hw_rev;			// hardware (pcb) revision 

	unsigned char	reg_cpu_ctrl;		// Reg 0x03   

	unsigned char		core_id;	/* SubSystem ID [0..7] */  
	unsigned char		core_rev;	/* SubSystem ID [8..15] */

	int				urbcount_read;
	int				urb_read_ind;
	
	int				urbcount_write;
	int				urb_write_ind;

	int				next_rx_ind;
	int				next_read_ind;

	int				next_tx_ind;
	int				next_write_ind;

	char			readchunk[MAX_USB_MODULES][AMF_USB_MAX_CHUNKSIZE * 2+1];
	char			writechunk[MAX_USB_MODULES][AMF_USB_MAX_CHUNKSIZE * 2+1];

	u8				regs[MAX_USB_MODULES][110];			// register shadow 

	u8				reg0shadow[MAX_USB_MODULES];		// read  fxs:68 fxo:5
	u8				reg1shadow[MAX_USB_MODULES];		// read  fxs:64 fxo:29
	u8				reg2shadow[MAX_USB_MODULES];		// read  fxs:64 fxo:29	 	?? 					 				
	u8				reg0shadow_write[MAX_USB_MODULES];	// write fxs:68 fxo:5 	
	int				reg0shadow_update[MAX_USB_MODULES];

	spinlock_t		cmd_lock;
	spinlock_t		lock;
	spinlock_t		tx_lock;

	char	idlebuf[MAX_USB_TX_LEN+1];

	struct tasklet_struct	bh_task;
		
	int		rx_sync;
	int		tx_sync; 

	unsigned char	ctrl_idle_pattern;
	unsigned long	tx_cmd_start;
	
	amf_skb_queue_t		tx_cmd_list;
	amf_skb_queue_t		tx_cmd_free_list;
	amf_skb_queue_t		rx_cmd_list;
	amf_skb_queue_t		rx_cmd_free_list;


	void		(*isr_func)(void*);
	void		*isr_arg;

	unsigned long		bhcount;
	unsigned long 		isrcount;
	unsigned long 		txcount;
	unsigned long 		rxcount;
	unsigned long 		dahdi_ticks;

	amf_usb_comm_err_stats_t	stats;

} amf_usb_t;


void wait_just_a_bit(int ms, int fast);
int amf_usb_init(void);
void amf_usb_exit(void);
void amf_usb_bh(unsigned long data);
int __amf_usb_cpu_write(amf_usb_t *ausb, unsigned char off, unsigned char data);
int __amf_usb_cpu_read(amf_usb_t *ausb, unsigned char off, unsigned char *data);
int __amf_usb_fxo_write(amf_usb_t *ausb, int mod_no, unsigned char off, unsigned char data);
u_int8_t __amf_usb_fxo_read(amf_usb_t *ausb, int mod_no, unsigned char off);
int amf_usb_start_transfer(struct amf_urb *aurb);
int amf_usb_rxtx_buffers_alloc(amf_usb_t *ausb);
int amf_usb_start_uart(amf_usb_t *ausb);
int amf_usb_stop_uart(amf_usb_t *ausb);
void amf_usb_set_urb_status_ready(amf_usb_t *ausb);
int amf_usb_set_config(amf_usb_t *usb, u8 request, int value, unsigned int *data, int size);
int amf_usb_set_config_single(amf_usb_t *usb, u8 request, int value, unsigned int data);
int amf_usb_get_config(amf_usb_t *usb, u8 request, int value, unsigned int *data, int size);
void amf_usb_set_transfer_urb(struct usb_interface *interface);
int amf_usb_search_rxsync(amf_usb_t *ausb);
int amf_usb_search_txsync(amf_usb_t *ausb);
int amf_usb_set_cpu(amf_usb_t *ausb);
int amf_usb_create(struct usb_interface*, int);
int amf_usb_destroy(struct usb_interface*);
int amf_usb_dahdi_register(amf_usb_t *ausb);
int amf_usb_dahdi_unregister(amf_usb_t *ausb);
int amf_usb_module_detect(amf_usb_t *);
void amf_check_hook(amf_usb_t *ausb, int mod_no);
void amf_voicedaa_recheck_sanity(amf_usb_t *ausb, int mod_no);
int amf_usb_init_daa(amf_usb_t *ausb, int mod_no, int fast, int sane);
int	amf_usb_set_opermode(amf_usb_t *ausb, char *opermode); 

int amf_usb_txdata_raw(amf_usb_t *ausb, unsigned char *data, int max_len);
int amf_usb_txdata_raw_ready(amf_usb_t *ausb);
int amf_usb_rxdata_raw(amf_usb_t *ausb, unsigned char *data, int max_len);
int amf_usb_rxurb_reset(amf_usb_t *ausb);
int amf_usb_txurb_reset(amf_usb_t *ausb);

void amf_usb_isr(void *arg);
int amf_usb_rxdata_enable(amf_usb_t *ausb, int enable);
int amf_usb_rxevent_enable(amf_usb_t *ausb, int mod_no, int enable);
int amf_usb_rxevent(amf_usb_t *ausb, int mod_no, int force);

int set_driver_started(void);
int get_driver_started(void);




//===================================== ENCODE / DECODE ===========================================

#define AMF_USB_CMD_TYPE_MASK		0x1C
#define AMF_USB_CMD_TYPE_SHIFT		2
#define AMF_USB_CMD_TYPE_READ_FXO	0x01
#define AMF_USB_CMD_TYPE_WRITE_FXO	0x02
#define AMF_USB_CMD_TYPE_EVENT		0x03
#define AMF_USB_CMD_TYPE_READ_CPU	0x04
#define AMF_USB_CMD_TYPE_WRITE_CPU	0x05
#define AMF_USB_CMD_TYPE_DECODE(ctrl)	(((ctrl) & AMF_USB_CMD_TYPE_MASK) >> AMF_USB_CMD_TYPE_SHIFT)
#define AMF_USB_CMD_TYPE_ENCODE(cmd)	((cmd) << AMF_USB_CMD_TYPE_SHIFT)

#define AMF_USB_CTRL_DECODE(buf)														\
						((buf)[0] & 0xC0) | (((buf)[1] & 0xC0) >> 2) |					\
						(((buf)[2] & 0xC0) >> 4) | (((buf)[3] & 0xC0) >> 6)

#define AMF_USB_CTRL_ENCODE(buf, ind)													\
	if (ind % 4 == 0 && ind % 32 == 0) *((buf)+ind) = (*((buf)+ind) & 0xCF) | 0x30;		\
	else if (ind % 4 == 0)  *((buf)+ind) = (*((buf)+ind) & 0xCF) | 0x10;

#define AMF_USB_CTRL_ENCODE1(buf, ctrl)													\
						*(buf)     = (*(buf) & 0xCF)     | (ctrl);

#define AMF_USB_DATA_ENCODE(buf, data)													\
						*(buf)     = (*(buf) & 0xF0)     | ((data) & 0x0F);				\
						*((buf)+2) = (*((buf)+2) & 0xF0) | (((data) >> 4) & 0x0F);
	
#define AMF_USB_CMD_ENCODE(buf, cmd)													\
						*(buf)     = (*(buf) & 0x3F)     | ((cmd) & 0xC0);				\
						*((buf)+1) = (*((buf)+1) & 0x3F) | (((cmd) & 0x30) << 2);		\
						*((buf)+2) = (*((buf)+2) & 0x3F) | (((cmd) & 0x0C) << 4);		\
						*((buf)+3) = (*((buf)+3) & 0x3F) | (((cmd) & 0x03) << 6); 


//===================================== ENCODE / DECODE. ===========================================



#endif  //__AMF_USB_H__
